#!/bin/bash
#
# Script for initializing and unsealing the Vault.
#
# Usage: ./scripts/init.sh
#

set -euo pipefail

# The guard below ensures, in most cases, that the script is executed from the correct directory (project directory).
# So it should be safe to assume that we can extract the path from the parent of the 'scripts' directory.
project_dir="$(dirname "$(dirname "$(realpath "$0")")")"

# ---------------- #
# Execution checks #
# ---------------- #
if [[ ! -f "docker-compose.yml" ]]; then
    echo >&2 "[ERROR] This script needs to be executed from the project directory!"
    exit 1
fi

if docker compose exec vault vault status | grep -q "Initialized.*true"; then
    echo "[INFO] Vault is already initialized. Skipping setup."
    exit 0
fi

# --------------------- #
# Environment variables #
# --------------------- #
env_file="${project_dir}/.env"

# Check if the .env file exists and source it.
if [[ -f "${env_file}" ]]; then
    echo "[INFO] Detected .env file, sourcing it."
    source "${env_file}"
fi

key_shares="${VAULT_KEY_SHARES:-7}"
key_threshold="${VAULT_KEY_THRESHOLD:-5}"

# ------------ #
# Run sequence #
# ------------ #
vault_dir="${project_dir}/docker/vault"
logs_dir="${vault_dir}/logs"
scripts_dir="${vault_dir}/scripts"

echo "[INFO] Initializing Vault."
docker compose exec vault vault operator init -key-shares="${key_shares}" -key-threshold="${key_threshold}" > "${logs_dir}/init.txt"

# Extract Unseal Keys & Root Token.
awk '/Unseal Key [0-9]+:/ {print $NF}' "${logs_dir}/init.txt" > "${logs_dir}/unseal-keys.txt"
root_token=$(awk '/Initial Root Token:/ {print $NF}' "${logs_dir}/init.txt")
echo "${root_token} # Root Token: Keep this secure!" > "${logs_dir}/tokens.txt"

# Unseal Vault.
count=1
for i in $(head -n "${key_threshold}" "${logs_dir}/unseal-keys.txt"); do
    echo "[INFO] Unsealing Vault (${count}/${key_threshold})..."
    docker compose exec vault vault operator unseal "$i" >/dev/null 2>&1
    ((count++))
done

# Secure permissions for the files containing the secrets.
for file in "${logs_dir}"/*.txt; do
  [[ -e "$file" ]] && chmod 600 "$file"
done

echo "[INFO] Vault Initialization complete."
